; *****************************************************************************************************
; 	PWM-Steuerung eines DC-Motors mit dem Tiny2313
; *****************************************************************************************************
;
;	Datei pwm.asm
; 	Autor: Jrgen Grzesina
; 	Stand: 12.03.09
;
; *****************************************************************************************************
; *****************************************************************************************************
;
.equ debuglevel = 0
;
; *****************************************************************************************************
	.equ takt = 8000000				; Quarzfrequenz

.nolist
.include "tn2313def.inc"
.include "arithmetik_header.asm"
.include "lcd-routines-bib_header.asm"
.include "rs232inc_header.asm"
.list



	.def temp1 = R16
	.def temp2 = R17
	.def temp3 = r18

	.def grubs	= R19				; zhlt bis 50
	.def jiffies = R20				; Hundertstelsekunden
	.def sekunden = R21
	.def Minuten = R22

	.def t1 = R23					; Zhler fr Tastenentprellung
	.def t2 = R24					; Zhler fr Tastenentprellung
	.def t3 = R25					; Zhler fr Tastenentprellung
	.def wahl = r26					; ausgesuchter Farbkanal
		.equ wrot = 0				; Bits fr die Farbwahl


	.equ flag = GPIOR0
		.equ frot = 2				; OC0A Pin 14 (PB2)



	.equ tastenport = portd			; Port fr den Tastenanschluss
	.equ tastenddr = ddrd			; DDR fr Tastenleitungen
	.equ tastenpin = pind	 		; Eingang fr Tastenabfrage
	.equ taste1 = 2 				; Funktionstaste Pin 6 (PD2)
	.equ taste2 = 3					; Auf-Taste Pin 7 (PD3)
	.equ taste3 = 4					; Ab-Taste Pin 8 (PD4)
	.equ longtime = 30				; Wert ab dem ein Langzeittastendruck gilt

	.equ ledport = portd			; Status LED: zustndiger Port
	.equ ledddr = ddrd				; 			  zustndiges ddr

	.equ r_port = portb
	.equ r_ddr = ddrb


	.equ rot = 2					; FarbLEDs und
	.equ LED = 6					; StatusLED

	.equ vocomp1b	= 200-1			; Jiffies entsprechen 0,01 s (1/(takt/8/200/50) = 1 Jiffie)
	
.dseg
	rot_wert:	.byte 1				; bergabevariablen der Farbwerte an



; *****************************************************************************************************

.cseg 
.org 0x0000
	rjmp start

.org 0x0005
	rjmp TOVL1						; Regelt GRN-bergabe

.org 0x0006
	rjmp TOVL0						; Regelt ROT- und BLAU-bergabe

.org 0x000c
	rjmp TOC1b						; erledigt die Zeitsteuerung

.org 0x001a
start:
	ldi temp1, ramend
	out spl, temp1

lcd_prep:
	; LCD initialisieren
	rcall lcd_init
	rcall lcd_clear
	rcall lcd_cursor
;	ldi temp1, 'T'
;	rcall lcd_data
.if debuglevel == 1 		; wir testen auf statische Signale (Kurzschlsse) 
	rjmp lcd_prep			; an den LCD-Pins
.endif

.if debuglevel == 2			; Wir prfen auf Bitmuster 0:1 an RS:E und
	cbi e_port, pin_e
	sbi rs_port, pin_rs
	in  temp1, lcd_port
	andi temp1, 0b00001111	; auf 1:0:1:0 auf PB7:PB6:PB5:PB4
	ori temp1, 0b10100000
	out lcd_port, temp1
test2:
	rjmp test2
.endif

.if debuglevel == 3			; Wir prfen auf Bitmuster 1:0 an RS:E und
	sbi e_port, pin_e
	cbi rs_port, pin_rs
	in  temp1, lcd_port
	andi temp1, 0b00001111	; auf 0:1:0:1 auf PB7:PB6:PB5:PB4
	ori temp1, 0b01010000
	out lcd_port, temp1
test3:
	rjmp test3
.endif

	; RS232 initialisieren
	rcall serinit
	ldi temp1, 0b00000100
	rcall hallo

.if debuglevel == 4
test4:
	rjmp v24_test
.endif

.if debuglevel == 5
test5:
	rcall serin_echo
	rjmp test5
.endif

	; Ports initialisieren - LCD-Leitungen managt "lcd_init"
	in temp1, tastenport
	andi temp1, ~((1<<taste1) | (1<<taste2) | (1<<taste3))	; explizit auf 0 setzen fr Eingang
	out tastenport, temp1									; interne Pullups aus
	
	in temp1, tastenddr
	andi temp1, ~((1<<taste1) | (1<<taste2) | (1<<taste3))	; explizit auf 0 setzen fr Eingang
	out tastenddr, temp1

	ldi t1, 0					; Tastenzhler auf Null
	ldi t2, 0
	ldi t3, 0
	ldi wahl, 1<<wrot			; bit fr roten Kanal
	

	sbi R_port, rot				; Rote LED ist lowaktiv, durch setzen auf 1 ausschalten
	sbi r_ddr, rot				; Leitung fr rote LED auf Ausgang
	sbi ledport, led			; dto
	sbi ledddr, led				

	; timer initialisieren
	; verwendet OC0B, OC0A, OC1A
	; alle Timer mit Vorteiler 8
	; OCR - Werte vorbesetzen
	ldi temp1, 3
	out OCR0A, temp1			; Steuerwert fr rot
	sts rot_wert, temp1
	rcall showrot
	ldi temp1, high(vocomp1b)
	out OCR1BH, temp1			; Highbyte der Vollstndigkeit wegen
	ldi temp1, low(vocomp1b)
	out OCR1BL, temp1			; Zeitsteuerung
	ldi temp1, 0
	out flag, temp1

	; TCCR0A: COM0A1, COM0A0, COM0B1, COM0B0,   -   ,    -    , WGM01, WGM00
	;         1       1       1       1        0        0       1      1
	; Fast PWM invertiert
	ldi temp1, 0b10110011	
	out tccr0a, temp1
	; TCCR0B: FOC0A,  FOC0B,    -   ,   -   , WGM02,  CS02,    CS01,   CS00
	;          0        0       0       0      0        0       1       0
	; Vorteiler 8
	ldi temp1, 0b00000010
	out tccr0b, temp1

	ldi temp1, 0
	out tccr1c, temp1			; kein Bedarf
	; TCCR1A: COM1A1, COM1A0, COM1B1, COM1B0,   -   ,    -    , WGM11, WGM10
	;          1       1       0       0        0       0       0      1
	ldi temp1, 0b11000001
	out tccr1a, temp1
	; TCCR1B: ICNC1,  ICES1,    -   , WGM13,  WGM12,  CS02,    CS01,   CS00
	;          0       0        0       0       1       0        1       0
	; no input capture, Vorteiler 8
	ldi temp1, 0b00001010
	out tccr1b, temp1

	; TIMSK:  TOIE1,  OCIE1A, OCIE1B,   -   , ICIE1, OCIE0B, TOIE0 , OCIE0A
	;    	   1        0        1      0       0       0      1       0
	ldi temp1, 0b10100010
	out timsk, temp1
	

	sei							; Interrupts freigeben

; *****************************************************************************************************

loop:
	sbic ucsra, rxc				; rxc-Bit = 0? dann keine taste gedrckt, tastenabfrage berspringen
	rcall tastenabfrage

jobs:

	rjmp loop					; Hauptschleife Ende

; *****************************************************************************************************
;	IRQ-Service-Routinen
; *****************************************************************************************************
; Timer0 berlauf
TOVL0:
	push temp1
	push temp2
	in temp2,sreg

w_rot:
	sbis flag, frot
	rjmp rb_done
	lds temp1, rot_wert
	out ocr0a, temp1
	cbi flag, frot
rb_done:
	out sreg, temp2
	pop temp2
	pop temp1
	reti

TOVL1:
	reti


TOC1B:
	push temp1
	push temp2
	in temp2,sreg
	push temp2
	
	inc grubs
	; ------------------------------- IRQ - Aufgaben ----------------------------------------------

	; Job 1   Zeitscheibe 1: Tastenabfrage ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
jif01:
	cpi grubs, 1					; 
	brne jif02
	; Taste 1 gedrckt, wenn taste1-bit=0  ........................................................
	sbic tastenpin, taste1			; berspringe nchsten Befehl, wenn Bit = 0
	rjmp t1r						; bit = 1 -> Taste nicht gedrckt
t1p:
	inc t1							; Zhler fr Taste 1 erhhen
	cpi t1, longtime+1				; Langzeit Tastendruck	
	brlo t1_ok
	ldi t1,longtime
t1_ok: 
	rjmp t1_fertig
t1r:
	clr t1							; Zhler fr Taste 1 auf 0, wenn nicht gedrckt
t1_fertig:

	; Taste 2 gedrckt, wenn taste2-bit=0 .........................................................
	sbic tastenpin, taste2			; berspringe nchsten Befehl, wenn Bit = 0
	rjmp t2r						; bit = 1 -> Taste nicht gedrckt
t2p:
	inc t2							; Zhler fr Taste 2 erhhen
	cpi t2, longtime+1				; Langzeit Tastendruck	
	brlo t2_ok
	ldi t2,longtime
t2_ok: 
	rjmp t2_fertig
t2r:
	clr t2							; Zhler fr Taste 1 auf 0, wenn nicht gedrckt
t2_fertig:

	; Taste 3 gedrckt, wenn taste3-bit=0 ........................................................
	sbic tastenpin, taste3			; berspringe nchsten Befehl, wenn Bit = 0
	rjmp t3r						; bit = 1 -> Taste nicht gedrckt
t3p:
	inc t3							; Zhler fr Taste 3 erhhen
	cpi t3, longtime+1				; Langzeit Tastendruck	
	brlo t3_ok
	ldi t3,longtime
t3_ok: 
	rjmp t3_fertig
t3r:
	clr t3							; Zhler fr Taste 1 auf 0, wenn nicht gedrckt
t3_fertig:

	; Job 2   Zeitscheibe 2: Tastenaktionen +++++++++++++++++++++++++++++++++++++++++++++++++++++
jif02:
	cpi grubs, 2					; 
	brne jif03
	; Taste 1 war kurz gedrckt, wenn t1=3
	ldi wahl, 1<<wrot				; bit fr roten Kanal setzen
	cpi t1,3
	brne t2k

t1b:
	rcall lcd_home					; Zeile 1 im LCD lschen
	ldi zl, low(str_hilfe*2)
	ldi zh, high(str_hilfe*2)
	rcall lcd_flash_string


t2k:
	cpi t2, 8 						; entsprechenden Kanal aufwrtszhlen
	brne t3k
	ldi t2, 0
	sbrc wahl, wrot
	rcall rotminus


t3k:
	cpi t3, 8 						; entsprechenden Kanal abwrtszhlen
	brne jif03
	ldi t3, 0
	sbrc wahl, wrot
	rcall rotplus
	
jif03:
	; ------------------------------- IRQ - Aufgaben  fertig ---------------------------------------
	cpi grubs, 50
	brne timing_done

	clr grubs
	inc jiffies
	cpi jiffies, 100
	brne timing_done

	clr jiffies
	inc sekunden
blink_LED:
	sbis ledport, led			; wenn Bit gesetzt, ist LED aus, rjmp berspringen zum Einschalten
	rjmp led_aus
led_an:
	cbi ledport, led			; Bit auf 0 -> LED an (lowaktiv)
	rjmp led_done
led_aus:
	sbi ledport, led			; Bit auf 1 -> LED aus
led_done:
	cpi sekunden, 60
	brne  timing_done

	clr sekunden
	inc minuten
	cpi minuten, 60
	brne  timing_done

	clr minuten

timing_done:
	pop temp2
	out sreg, temp2
	pop temp2
	pop temp1
	reti

; *****************************************************************************************************
;	Bibliotheksroutinen
; *****************************************************************************************************

.nolist
.include "arithmetik.asm"
.include "lcd-routines-bib.asm"
.include "rs232inc.asm"
.list

; *****************************************************************************************************
;	Jobroutinen
; *****************************************************************************************************

Hallo:
	ldi zl, low(2*str_hallo)
	ldi zh, high(2*str_hallo)
	rcall v24_flash_string
	rcall v24_lfcr
	ldi zl, low(2*str_hallo)
	ldi zh, high(2*str_hallo)
	rcall lcd_flash_string
	ret
; -----------------------------------------------------------------------------------------------------
tastenabfrage:
	rcall serin_echo
	;  Im Moment ungenutzt
	cpi temp1, 'b'
	brne ta01
	rjmp ta99
ta01:
	; Rotwert senken
	cpi temp1, 'r'
	brne ta02
	rcall rotminus
	rjmp ta99
ta02:
	;  Im Moment ungenutzt
	cpi temp1, 'g'
	brne ta03
	rjmp ta99
ta03:
	;  Im Moment ungenutzt
	cpi temp1, 'G'
	brne ta04
	rjmp ta99
ta04:
	; Rotwert steigern
	cpi temp1, 'R'
	brne ta05
	rcall rotplus
	rjmp ta99
ta05:
	;  Im Moment ungenutzt
	cpi temp1, 'B'
	brne ta06
	rjmp ta99
ta06:
	; null
	cpi temp1, 'n'
	brne ta07
	RCALL null
	rjmp ta99
ta07:
	; voll
	cpi temp1, 'v'
	brne ta08
	ldi temp1, 255
	sts rot_wert,temp1
	rcall showrot
	sbi flag, frot
	rjmp ta99
ta08:
	; Im Moment ungenutzt
	cpi temp1, 'a'
	brne ta09
	rjmp ta99

ta09:
.if debuglevel == 6
	cpi temp1, 'i'
	brne ta10
	rcall lcd_init
	rjmp ta99	
ta10: 
	cpi temp1, 'c'
	brne ta11
	rcall lcd_clear
	rjmp ta99
ta11:
	cpi temp1, 'h'
	brne ta12
	rcall lcd_home
	rjmp ta99
ta12:


.endif
ta99:
	ret

showrot:
	push temp1
	in temp1, sreg
	push temp1
	push temp2

	ldi temp1, 0
	ldi temp2, 1
	rcall lcd_position
	lds temp1, rot_wert
	rcall lcd_number

	pop temp2
	pop temp1
	out sreg, temp1
	pop temp1
	ret


null:
	ldi temp1, 1
	sts rot_wert,temp1
	rcall showrot
	sbi flag, frot
	ret


rotminus:
	push temp1
	push temp2
	in temp2,sreg
	lds temp1, rot_wert
	cpi temp1, 1
	brsh rminus
	rjmp rm99
rminus:
	dec temp1
	sts rot_wert,temp1
	sbi flag, frot
	rcall showrot
rm99:
	out sreg, temp2
	pop temp2
	pop temp1
	ret

rotplus:
	push temp1
	push temp2
	in temp2,sreg
	lds temp1, rot_wert
	cpi temp1, 255
	brne rplus
	rjmp rp99
rplus:
	inc temp1
	sts rot_wert,temp1
	sbi flag, frot
	rcall showrot
rp99:
	out sreg, temp2
	pop temp2
	pop temp1
	ret




; *****************************************************************************************************
;	Konstanten
; *****************************************************************************************************

str_hallo:
;        0123456789ABCDEF
	.db "Motor-Dimmer   ", 0

str_hilfe:
	.db "  ^ T2 - v T3  ", 0

str_blank_line: 
	.db "               ", 0
.exit


TOVL1:
	push temp1
	push temp2
	in temp2,sreg

	sbis flag, fgruen
	rjmp g_done
	lds temp1, gruen_wert
	out ocr1al, temp1
	cbi flag, fgruen
g_done:
	out sreg, temp2
	pop temp2
	pop temp1
	reti


